import pytest
from orchestrator.types import SubscriptionLifecycle

from gso.products.product_blocks.bgp_session import IPFamily
from gso.products.product_types.nren_l3_core_service import ImportedNRENL3CoreService, NRENL3CoreServiceType
from gso.utils.shared_enums import SBPType
from test.workflows import assert_complete, extract_state, run_workflow


@pytest.mark.parametrize("l3_core_service_type", [NRENL3CoreServiceType.GEANT_IP, NRENL3CoreServiceType.IAS])
def test_create_imported_nren_l3_core_service_success(
    faker, partner_factory, edge_port_subscription_factory, l3_core_service_type
):
    creation_form_input_data = {
        "partner": partner_factory()["name"],
        "service_type": l3_core_service_type,
        "service_binding_ports": [
            {
                "edge_port": edge_port_subscription_factory(),
                "ap_type": "PRIMARY",
                "geant_sid": faker.geant_sid(),
                "sbp_type": SBPType.L3,
                "is_tagged": faker.boolean(),
                "vlan_id": faker.vlan_id(),
                "ipv4_address": faker.ipv4(),
                "ipv4_mask": faker.ipv4_netmask(),
                "ipv6_address": faker.ipv6(),
                "ipv6_mask": faker.ipv6_netmask(),
                "custom_firewall_filters": faker.boolean(),
                "bgp_peers": [
                    {
                        "bfd_enabled": faker.boolean(),
                        "bfd_interval": faker.pyint(),
                        "bfd_multiplier": faker.pyint(),
                        "has_custom_policies": faker.boolean(),
                        "authentication_key": faker.password(),
                        "multipath_enabled": faker.boolean(),
                        "send_default_route": faker.boolean(),
                        "is_passive": faker.boolean(),
                        "peer_address": faker.ipv4(),
                        "families": [IPFamily.V4UNICAST, IPFamily.V4MULTICAST],
                        "is_multi_hop": faker.boolean(),
                        "rtbh_enabled": faker.boolean(),
                    },
                    {
                        "bfd_enabled": faker.boolean(),
                        "bfd_interval": faker.pyint(),
                        "bfd_multiplier": faker.pyint(),
                        "has_custom_policies": faker.boolean(),
                        "authentication_key": faker.password(),
                        "multipath_enabled": faker.boolean(),
                        "send_default_route": faker.boolean(),
                        "is_passive": faker.boolean(),
                        "peer_address": faker.ipv6(),
                        "families": [IPFamily.V6UNICAST],
                        "is_multi_hop": faker.boolean(),
                        "rtbh_enabled": faker.boolean(),
                    },
                ],
            }
        ],
    }

    result, _, _ = run_workflow("create_imported_nren_l3_core_service", [creation_form_input_data])
    state = extract_state(result)
    subscription = ImportedNRENL3CoreService.from_subscription(state["subscription_id"])
    assert_complete(result)
    assert subscription.status == SubscriptionLifecycle.ACTIVE
